@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2015 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -513,9 +513,9 @@ public:
         copyFrom(that.raw());
     }
 
-    Utf8Str(CBSTR that)
+    Utf8Str(CBSTR that, size_t a_cwcSize = RTSTR_MAX)
     {
-        copyFrom(that);
+        copyFrom(that, a_cwcSize);
     }
 
     Utf8Str(const char *a_pszSrc, size_t a_cchSrc)
@@ -564,8 +564,6 @@ public:
         return *this;
     }
 
-    bool operator<(const RTCString &that) const { return RTCString::operator<(that); }
-
     /**
      * Extended assignment method that returns a COM status code instead of an
      * exception on failure.
@@ -575,7 +573,7 @@ public:
      */
     HRESULT assignEx(Utf8Str const &a_rSrcStr)
     {
-        return copyFromExNComRC(a_rSrcStr.m_psz, a_rSrcStr.m_cch);
+        return copyFromExNComRC(a_rSrcStr.m_psz, 0, a_rSrcStr.m_cch);
     }
 
     /**
@@ -593,7 +591,7 @@ public:
         if (   a_offSrc + a_cchSrc > a_rSrcStr.m_cch
             || a_offSrc > a_rSrcStr.m_cch)
             return E_INVALIDARG;
-        return copyFromExNComRC(a_rSrcStr.m_psz, a_rSrcStr.m_cch);
+        return copyFromExNComRC(a_rSrcStr.m_psz, a_offSrc, a_cchSrc);
     }
 
     /**
@@ -605,7 +603,7 @@ public:
      */
     HRESULT assignEx(const char *a_pcszSrc)
     {
-        return copyFromExNComRC(a_pcszSrc, a_pcszSrc ? strlen(a_pcszSrc) : 0);
+        return copyFromExNComRC(a_pcszSrc, 0, a_pcszSrc ? strlen(a_pcszSrc) : 0);
     }
 
     /**
@@ -619,7 +617,7 @@ public:
      */
     HRESULT assignEx(const char *a_pcszSrc, size_t a_cchSrc)
     {
-        return copyFromExNComRC(a_pcszSrc, a_cchSrc);
+        return copyFromExNComRC(a_pcszSrc, 0, a_cchSrc);
     }
 
     RTMEMEF_NEW_AND_DELETE_OPERATORS();
@@ -705,10 +703,13 @@ public:
     Utf8Str& stripPath();
 
     /**
-     * Removes a trailing file name extension from the member string, if present.
-     * Calls RTPathStripExt() without having to mess with mutableRaw().
+     * Removes a trailing file name suffix from the member string, if present.
+     * Calls RTPathStripSuffix() without having to mess with mutableRaw().
      */
-    Utf8Str& stripExt();
+    Utf8Str& stripSuffix();
+
+    // Parse key=value pairs from string
+    size_t parseKeyValue(Utf8Str &key, Utf8Str &value, size_t pos = 0, const Utf8Str &pairSeparator = ",", const Utf8Str &keyValueSeparator = "=") const;
 
     /**
      *  Static immutable empty-string object. May be used for comparison purposes.
@@ -716,9 +717,9 @@ public:
     static const Utf8Str Empty;
 protected:
 
-    void copyFrom(CBSTR a_pbstr);
+    void copyFrom(CBSTR a_pbstr, size_t a_cwcMax = RTSTR_MAX);
     HRESULT copyFromEx(CBSTR a_pbstr);
-    HRESULT copyFromExNComRC(const char *a_pcszSrc, size_t a_cchSrc);
+    HRESULT copyFromExNComRC(const char *a_pcszSrc, size_t a_offSrc, size_t a_cchSrc);
 
     friend class Bstr; /* to access our raw_copy() */
 };
