@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2015 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -58,6 +58,8 @@ typedef enum MMTAG
     MM_TAG_CSAM_PATCH,
 
     MM_TAG_CPUM_CTX,
+    MM_TAG_CPUM_CPUID,
+    MM_TAG_CPUM_MSRS,
 
     MM_TAG_DBGF,
     MM_TAG_DBGF_AS,
@@ -109,6 +111,7 @@ typedef enum MMTAG
     MM_TAG_PGM,
     MM_TAG_PGM_CHUNK_MAPPING,
     MM_TAG_PGM_HANDLERS,
+    MM_TAG_PGM_HANDLER_TYPES,
     MM_TAG_PGM_MAPPINGS,
     MM_TAG_PGM_PHYS,
     MM_TAG_PGM_POOL,
@@ -130,7 +133,7 @@ typedef enum MMTAG
 
     MM_TAG_VMM,
 
-    MM_TAG_HWACCM,
+    MM_TAG_HM,
 
     MM_TAG_32BIT_HACK = 0x7fffffff
 } MMTAG;
@@ -139,7 +142,6 @@ typedef enum MMTAG
 
 
 /** @defgroup grp_mm_hyper  Hypervisor Memory Management
- * @ingroup grp_mm
  * @{ */
 
 VMMDECL(RTR3PTR)    MMHyperR0ToR3(PVM pVM, RTR0PTR R0Ptr);
@@ -205,6 +207,7 @@ DECLINLINE(RTRCPTR) MMHyperCCToRC(PVM pVM, void *pv)
 
 
 VMMDECL(int)        MMHyperAlloc(PVM pVM, size_t cb, uint32_t uAlignment, MMTAG enmTag, void **ppv);
+VMMDECL(int)        MMHyperDupMem(PVM pVM, const void *pvSrc, size_t cb, unsigned uAlignment, MMTAG enmTag, void **ppv);
 VMMDECL(int)        MMHyperFree(PVM pVM, void *pv);
 VMMDECL(void)       MMHyperHeapCheck(PVM pVM);
 VMMDECL(int)        MMR3LockCall(PVM pVM);
@@ -213,6 +216,8 @@ VMMDECL(void)       MMHyperHeapDump(PVM pVM);
 #endif
 VMMDECL(size_t)     MMHyperHeapGetFreeSize(PVM pVM);
 VMMDECL(size_t)     MMHyperHeapGetSize(PVM pVM);
+VMMDECL(void *)     MMHyperHeapOffsetToPtr(PVM pVM, uint32_t offHeap);
+VMMDECL(uint32_t)   MMHyperHeapPtrToOffset(PVM pVM, void *pv);
 VMMDECL(RTGCPTR)    MMHyperGetArea(PVM pVM, size_t *pcb);
 VMMDECL(bool)       MMHyperIsInsideArea(PVM pVM, RTGCPTR GCPtr);
 
@@ -237,9 +242,8 @@ VMMDECL(int)        MMPagePhys2PageTry(PVM pVM, RTHCPHYS HCPhysPage, void **ppvP
 /** @} */
 
 
-#ifdef IN_RING3
+#if defined(IN_RING3) || defined(DOXYGEN_RUNNING)
 /** @defgroup grp_mm_r3    The MM Host Context Ring-3 API
- * @ingroup grp_mm
  * @{
  */
 
@@ -249,6 +253,7 @@ VMMR3DECL(int)      MMR3InitPaging(PVM pVM);
 VMMR3DECL(int)      MMR3HyperInitFinalize(PVM pVM);
 VMMR3DECL(int)      MMR3Term(PVM pVM);
 VMMR3DECL(void)     MMR3TermUVM(PUVM pUVM);
+VMMR3_INT_DECL(bool) MMR3IsInitialized(PVM pVM);
 VMMR3DECL(int)      MMR3ReserveHandyPages(PVM pVM, uint32_t cHandyPages);
 VMMR3DECL(int)      MMR3IncreaseBaseReservation(PVM pVM, uint64_t cAddBasePages);
 VMMR3DECL(int)      MMR3AdjustFixedReservation(PVM pVM, int32_t cDeltaFixedPages, const char *pszDesc);
@@ -257,10 +262,10 @@ VMMR3DECL(int)      MMR3UpdateShadowReservation(PVM pVM, uint32_t cShadowPages);
 VMMR3DECL(int)      MMR3HCPhys2HCVirt(PVM pVM, RTHCPHYS HCPhys, void **ppv);
 
 /** @defgroup grp_mm_r3_hyper  Hypervisor Memory Manager (HC R3 Portion)
- * @ingroup grp_mm_r3
  * @{ */
 VMMR3DECL(int)      MMR3HyperAllocOnceNoRel(PVM pVM, size_t cb, uint32_t uAlignment, MMTAG enmTag, void **ppv);
 VMMR3DECL(int)      MMR3HyperAllocOnceNoRelEx(PVM pVM, size_t cb, uint32_t uAlignment, MMTAG enmTag, uint32_t fFlags, void **ppv);
+VMMR3DECL(int)      MMR3HyperRealloc(PVM pVM, void *pv, size_t cb, unsigned uAlignmentNew, MMTAG enmTagNew, size_t cbNew, void **ppv);
 /** @name  MMR3HyperAllocOnceNoRelEx flags
  * @{ */
 /** Must have kernel mapping.
@@ -284,14 +289,12 @@ VMMR3DECL(int)      MMR3HyperReadGCVirt(PVM pVM, void *pvDst, RTGCPTR GCPtr, siz
 
 /** @defgroup grp_mm_phys   Guest Physical Memory Manager
  * @todo retire this group, elimintating or moving MMR3PhysGetRamSize to PGMPhys.
- * @ingroup grp_mm_r3
  * @{ */
 VMMR3DECL(uint64_t) MMR3PhysGetRamSize(PVM pVM);
 /** @} */
 
 
 /** @defgroup grp_mm_page   Physical Page Pool
- * @ingroup grp_mm_r3
  * @{ */
 VMMR3DECL(void *)   MMR3PageAlloc(PVM pVM);
 VMMR3DECL(RTHCPHYS) MMR3PageAllocPhys(PVM pVM);
@@ -305,7 +308,6 @@ VMMR3DECL(RTHCPHYS) MMR3PageDummyHCPhys(PVM pVM);
 
 
 /** @defgroup grp_mm_heap   Heap Manager
- * @ingroup grp_mm_r3
  * @{ */
 VMMR3DECL(void *)   MMR3HeapAlloc(PVM pVM, MMTAG enmTag, size_t cbSize);
 VMMR3DECL(void *)   MMR3HeapAllocU(PUVM pUVM, MMTAG enmTag, size_t cbSize);
@@ -325,8 +327,7 @@ VMMR3DECL(char *)   MMR3HeapAPrintfVU(PUVM pUVM, MMTAG enmTag, const char *pszFo
 VMMR3DECL(void)     MMR3HeapFree(void *pv);
 /** @} */
 
-/** @defgroup grp_mm_heap   User-kernel Heap Manager.
- * @ingroup grp_mm_r3
+/** @defgroup grp_mm_ukheap   User-kernel Heap Manager.
  *
  * The memory is safely accessible from kernel context as well as user land.
  *
@@ -339,13 +340,12 @@ VMMR3DECL(void)     MMR3UkHeapFree(PVM pVM, void *pv, MMTAG enmTag);
 /** @} */
 
 /** @} */
-#endif /* IN_RING3 */
+#endif /* IN_RING3 || DOXYGEN_RUNNING */
 
 
 
-#ifdef IN_RC
-/** @defgroup grp_mm_gc    The MM Guest Context API
- * @ingroup grp_mm
+#if defined(IN_RC) || defined(DOXYGEN_RUNNING)
+/** @defgroup grp_mm_rc    The MM Raw-mode Context API
  * @{
  */
 
@@ -360,7 +360,7 @@ VMMRCDECL(int)      MMGCRamRead(PVM pVM, void *pDst, void *pSrc, size_t cb);
 VMMRCDECL(int)      MMGCRamWrite(PVM pVM, void *pDst, void *pSrc, size_t cb);
 
 /** @} */
-#endif /* IN_RC */
+#endif /* IN_RC || DOXYGEN_RUNNING */
 
 /** @} */
 RT_C_DECLS_END
