@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2015 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -456,7 +456,7 @@ RT_C_DECLS_END
 #if defined(RTASSERT_QUIET) && !defined(DOXYGEN_RUNNING)
 # define RTAssertMsg1Weak(pszExpr, uLine, pszfile, pszFunction) \
                                 do { } while (0)
-# define RTAssertMsg2Weak       if (0) RTAssertMsg2Weak
+# define RTAssertMsg2Weak       if (1) {} else RTAssertMsg2Weak
 #endif
 
 /** @def RTAssertDoPanic
@@ -498,9 +498,11 @@ RT_C_DECLS_END
 #ifdef RT_STRICT
 # define Assert(expr)  \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
-            RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
+            RTAssertMsg1Weak(#expr, __LINE__, __FILE__, RT_GCC_EXTENSION __PRETTY_FUNCTION__); \
             RTAssertPanic(); \
         } \
     } while (0)
@@ -518,9 +520,11 @@ RT_C_DECLS_END
 #ifdef RT_STRICT
 # define AssertStmt(expr, stmt)  \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
-            RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
+            RTAssertMsg1Weak(#expr, __LINE__, __FILE__, RT_GCC_EXTENSION __PRETTY_FUNCTION__); \
             RTAssertPanic(); \
             stmt; \
         } \
@@ -528,7 +532,9 @@ RT_C_DECLS_END
 #else
 # define AssertStmt(expr, stmt)  \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
             stmt; \
         } \
@@ -546,9 +552,11 @@ RT_C_DECLS_END
 #ifdef RT_STRICT
 # define AssertReturn(expr, rc) \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
-            RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
+            RTAssertMsg1Weak(#expr, __LINE__, __FILE__, RT_GCC_EXTENSION __PRETTY_FUNCTION__); \
             RTAssertPanic(); \
             return (rc); \
         } \
@@ -556,7 +564,9 @@ RT_C_DECLS_END
 #else
 # define AssertReturn(expr, rc) \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
             return (rc); \
     } while (0)
 #endif
@@ -575,7 +585,9 @@ RT_C_DECLS_END
 #ifdef RT_STRICT
 # define AssertReturnStmt(expr, stmt, rc) \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
             RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
             RTAssertPanic(); \
@@ -586,7 +598,9 @@ RT_C_DECLS_END
 #else
 # define AssertReturnStmt(expr, stmt, rc) \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
             stmt; \
             return (rc); \
@@ -603,9 +617,11 @@ RT_C_DECLS_END
 #ifdef RT_STRICT
 # define AssertReturnVoid(expr) \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
-            RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
+            RTAssertMsg1Weak(#expr, __LINE__, __FILE__, RT_GCC_EXTENSION __PRETTY_FUNCTION__); \
             RTAssertPanic(); \
             return; \
         } \
@@ -613,7 +629,9 @@ RT_C_DECLS_END
 #else
 # define AssertReturnVoid(expr) \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
             return; \
     } while (0)
 #endif
@@ -630,7 +648,9 @@ RT_C_DECLS_END
 #ifdef RT_STRICT
 # define AssertReturnVoidStmt(expr, stmt) \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
             RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
             RTAssertPanic(); \
@@ -641,7 +661,9 @@ RT_C_DECLS_END
 #else
 # define AssertReturnVoidStmt(expr, stmt) \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
             stmt; \
             return; \
@@ -658,7 +680,9 @@ RT_C_DECLS_END
  */
 #ifdef RT_STRICT
 # define AssertBreak(expr) \
-    if (RT_UNLIKELY(!(expr))) \
+    if (RT_LIKELY(!!(expr))) \
+    { /* likely */ } \
+    else if (1) \
     { \
         RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
         RTAssertPanic(); \
@@ -666,9 +690,10 @@ RT_C_DECLS_END
     } else do {} while (0)
 #else
 # define AssertBreak(expr) \
-    if (RT_UNLIKELY(!(expr))) \
-        break; \
-    else do {} while (0)
+    if (RT_LIKELY(!!(expr))) \
+    { /* likely */ } \
+    else \
+        break
 #endif
 
 /** @def AssertBreakStmt
@@ -680,7 +705,10 @@ RT_C_DECLS_END
  */
 #ifdef RT_STRICT
 # define AssertBreakStmt(expr, stmt) \
-    if (RT_UNLIKELY(!(expr))) { \
+    if (RT_LIKELY(!!(expr))) \
+    { /* likely */ } \
+    else if (1) \
+    { \
         RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
         RTAssertPanic(); \
         stmt; \
@@ -688,7 +716,10 @@ RT_C_DECLS_END
     } else do {} while (0)
 #else
 # define AssertBreakStmt(expr, stmt) \
-    if (RT_UNLIKELY(!(expr))) { \
+    if (RT_LIKELY(!!(expr))) \
+    { /* likely */ } \
+    else if (1) \
+    { \
         stmt; \
         break; \
     } else do {} while (0)
@@ -703,9 +734,11 @@ RT_C_DECLS_END
 #ifdef RT_STRICT
 # define AssertMsg(expr, a)  \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
-            RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
+            RTAssertMsg1Weak(#expr, __LINE__, __FILE__, RT_GCC_EXTENSION __PRETTY_FUNCTION__); \
             RTAssertMsg2Weak a; \
             RTAssertPanic(); \
         } \
@@ -727,7 +760,9 @@ RT_C_DECLS_END
 #ifdef RT_STRICT
 # define AssertMsgStmt(expr, a, stmt)  \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
             RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
             RTAssertMsg2Weak a; \
@@ -738,7 +773,9 @@ RT_C_DECLS_END
 #else
 # define AssertMsgStmt(expr, a, stmt)  \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
             stmt; \
         } \
@@ -756,9 +793,11 @@ RT_C_DECLS_END
 #ifdef RT_STRICT
 # define AssertMsgReturn(expr, a, rc)  \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
-            RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
+            RTAssertMsg1Weak(#expr, __LINE__, __FILE__, RT_GCC_EXTENSION __PRETTY_FUNCTION__); \
             RTAssertMsg2Weak a; \
             RTAssertPanic(); \
             return (rc); \
@@ -767,7 +806,9 @@ RT_C_DECLS_END
 #else
 # define AssertMsgReturn(expr, a, rc) \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
             return (rc); \
     } while (0)
 #endif
@@ -787,7 +828,9 @@ RT_C_DECLS_END
 #ifdef RT_STRICT
 # define AssertMsgReturnStmt(expr, a, stmt, rc)  \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
             RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
             RTAssertMsg2Weak a; \
@@ -799,7 +842,9 @@ RT_C_DECLS_END
 #else
 # define AssertMsgReturnStmt(expr, a, stmt, rc) \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
             stmt; \
             return (rc); \
@@ -817,9 +862,11 @@ RT_C_DECLS_END
 #ifdef RT_STRICT
 # define AssertMsgReturnVoid(expr, a)  \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
-            RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
+            RTAssertMsg1Weak(#expr, __LINE__, __FILE__, RT_GCC_EXTENSION __PRETTY_FUNCTION__); \
             RTAssertMsg2Weak a; \
             RTAssertPanic(); \
             return; \
@@ -828,7 +875,9 @@ RT_C_DECLS_END
 #else
 # define AssertMsgReturnVoid(expr, a) \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
             return; \
     } while (0)
 #endif
@@ -841,12 +890,14 @@ RT_C_DECLS_END
  *
  * @param   expr    Expression which should be true.
  * @param   a       printf argument list (in parenthesis).
- * @param   stmt    Statement to execute before break in case of a failed assertion.
+ * @param   stmt    Statement to execute before return in case of a failed assertion.
  */
 #ifdef RT_STRICT
 # define AssertMsgReturnVoidStmt(expr, a, stmt)  \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
             RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
             RTAssertMsg2Weak a; \
@@ -858,7 +909,9 @@ RT_C_DECLS_END
 #else
 # define AssertMsgReturnVoidStmt(expr, a, stmt) \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
             stmt; \
             return; \
@@ -875,8 +928,10 @@ RT_C_DECLS_END
  * @param   a       printf argument list (in parenthesis).
  */
 #ifdef RT_STRICT
-# define AssertMsgBreak(expr, a)  \
-    if (RT_UNLIKELY(!(expr))) \
+# define AssertMsgBreak(expr, a) \
+    if (RT_LIKELY(!!(expr))) \
+    { /* likely */ } \
+    else if (1) \
     { \
         RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
         RTAssertMsg2Weak a; \
@@ -885,9 +940,10 @@ RT_C_DECLS_END
     } else do {} while (0)
 #else
 # define AssertMsgBreak(expr, a) \
-    if (RT_UNLIKELY(!(expr))) \
-        break; \
-    else do {} while (0)
+    if (RT_LIKELY(!!(expr))) \
+    { /* likely */ } \
+    else \
+        break
 #endif
 
 /** @def AssertMsgBreakStmt
@@ -900,7 +956,10 @@ RT_C_DECLS_END
  */
 #ifdef RT_STRICT
 # define AssertMsgBreakStmt(expr, a, stmt) \
-    if (RT_UNLIKELY(!(expr))) { \
+    if (RT_LIKELY(!!(expr))) \
+    { /* likely */ } \
+    else if (1) \
+    { \
         RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
         RTAssertMsg2Weak a; \
         RTAssertPanic(); \
@@ -909,7 +968,10 @@ RT_C_DECLS_END
     } else do {} while (0)
 #else
 # define AssertMsgBreakStmt(expr, a, stmt) \
-    if (RT_UNLIKELY(!(expr))) { \
+    if (RT_LIKELY(!!(expr))) \
+    { /* likely */ } \
+    else if (1) \
+    { \
         stmt; \
         break; \
     } else do {} while (0)
@@ -921,7 +983,7 @@ RT_C_DECLS_END
 #ifdef RT_STRICT
 # define AssertFailed()  \
     do { \
-        RTAssertMsg1Weak((const char *)0, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
+        RTAssertMsg1Weak((const char *)0, __LINE__, __FILE__, RT_GCC_EXTENSION __PRETTY_FUNCTION__); \
         RTAssertPanic(); \
     } while (0)
 #else
@@ -1058,7 +1120,7 @@ RT_C_DECLS_END
 #ifdef RT_STRICT
 # define AssertMsgFailed(a)  \
     do { \
-        RTAssertMsg1Weak((const char *)0, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
+        RTAssertMsg1Weak((const char *)0, __LINE__, __FILE__, RT_GCC_EXTENSION __PRETTY_FUNCTION__); \
         RTAssertMsg2Weak a; \
         RTAssertPanic(); \
     } while (0)
@@ -1195,7 +1257,9 @@ RT_C_DECLS_END
  */
 #define AssertLogRel(expr) \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
             RTAssertLogRelMsg1(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
             RTAssertPanic(); \
@@ -1211,7 +1275,9 @@ RT_C_DECLS_END
  */
 #define AssertLogRelReturn(expr, rc) \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
             RTAssertLogRelMsg1(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
             RTAssertPanic(); \
@@ -1227,7 +1293,9 @@ RT_C_DECLS_END
  */
 #define AssertLogRelReturnVoid(expr) \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
             RTAssertLogRelMsg1(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
             RTAssertPanic(); \
@@ -1242,7 +1310,9 @@ RT_C_DECLS_END
  * @param   expr    Expression which should be true.
  */
 #define AssertLogRelBreak(expr) \
-    if (RT_UNLIKELY(!(expr))) \
+    if (RT_LIKELY(!!(expr))) \
+    { /* likely */ } \
+    else if (1) \
     { \
         RTAssertLogRelMsg1(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
         RTAssertPanic(); \
@@ -1258,7 +1328,9 @@ RT_C_DECLS_END
  * @param   stmt    Statement to execute before break in case of a failed assertion.
  */
 #define AssertLogRelBreakStmt(expr, stmt) \
-    if (RT_UNLIKELY(!(expr))) \
+    if (RT_LIKELY(!!(expr))) \
+    { /* likely */ } \
+    else if (1) \
     { \
         RTAssertLogRelMsg1(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
         RTAssertPanic(); \
@@ -1275,7 +1347,9 @@ RT_C_DECLS_END
  */
 #define AssertLogRelMsg(expr, a) \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else\
         { \
             RTAssertLogRelMsg1(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
             RTAssertLogRelMsg2(a); \
@@ -1293,7 +1367,9 @@ RT_C_DECLS_END
  */
 #define AssertLogRelMsgStmt(expr, a, stmt) \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else\
         { \
             RTAssertLogRelMsg1(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
             RTAssertLogRelMsg2(a); \
@@ -1312,7 +1388,9 @@ RT_C_DECLS_END
  */
 #define AssertLogRelMsgReturn(expr, a, rc) \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else\
         { \
             RTAssertLogRelMsg1(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
             RTAssertLogRelMsg2(a); \
@@ -1334,7 +1412,9 @@ RT_C_DECLS_END
  */
 #define AssertLogRelMsgReturnStmt(expr, a, stmt, rcRet) \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else\
         { \
             RTAssertLogRelMsg1(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
             RTAssertLogRelMsg2(a); \
@@ -1353,7 +1433,9 @@ RT_C_DECLS_END
  */
 #define AssertLogRelMsgReturnVoid(expr, a) \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else\
         { \
             RTAssertLogRelMsg1(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
             RTAssertLogRelMsg2(a); \
@@ -1370,7 +1452,9 @@ RT_C_DECLS_END
  * @param   a       printf argument list (in parenthesis).
  */
 #define AssertLogRelMsgBreak(expr, a) \
-    if (RT_UNLIKELY(!(expr))) \
+    if (RT_LIKELY(!!(expr))) \
+    { /* likely */ } \
+    else if (1) \
     { \
         RTAssertLogRelMsg1(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
         RTAssertLogRelMsg2(a); \
@@ -1388,7 +1472,9 @@ RT_C_DECLS_END
  * @param   stmt    Statement to execute before break in case of a failed assertion.
  */
 #define AssertLogRelMsgBreakStmt(expr, a, stmt) \
-    if (RT_UNLIKELY(!(expr))) \
+    if (RT_LIKELY(!!(expr))) \
+    { /* likely */ } \
+    else if (1) \
     { \
         RTAssertLogRelMsg1(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
         RTAssertLogRelMsg2(a); \
@@ -1611,9 +1697,11 @@ RT_C_DECLS_END
  */
 #define AssertRelease(expr)  \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
-            RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
+            RTAssertMsg1Weak(#expr, __LINE__, __FILE__, RT_GCC_EXTENSION __PRETTY_FUNCTION__); \
             RTAssertReleasePanic(); \
         } \
     } while (0)
@@ -1626,7 +1714,9 @@ RT_C_DECLS_END
  */
 #define AssertReleaseReturn(expr, rc)  \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
             RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
             RTAssertReleasePanic(); \
@@ -1641,7 +1731,9 @@ RT_C_DECLS_END
  */
 #define AssertReleaseReturnVoid(expr)  \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
             RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
             RTAssertReleasePanic(); \
@@ -1656,13 +1748,13 @@ RT_C_DECLS_END
  * @param   expr    Expression which should be true.
  */
 #define AssertReleaseBreak(expr)  \
-    if { \
-        if (RT_UNLIKELY(!(expr))) \
-        { \
-            RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
-            RTAssertReleasePanic(); \
-            break; \
-        } \
+    if (RT_LIKELY(!!(expr))) \
+    { /* likely */ } \
+    else if (1) \
+    { \
+        RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
+        RTAssertReleasePanic(); \
+        break; \
     } else do {} while (0)
 
 /** @def AssertReleaseBreakStmt
@@ -1672,7 +1764,9 @@ RT_C_DECLS_END
  * @param   stmt    Statement to execute before break in case of a failed assertion.
  */
 #define AssertReleaseBreakStmt(expr, stmt)  \
-    if (RT_UNLIKELY(!(expr))) \
+    if (RT_LIKELY(!!(expr))) \
+    { /* likely */ } \
+    else if (1) \
     { \
         RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
         RTAssertReleasePanic(); \
@@ -1689,7 +1783,9 @@ RT_C_DECLS_END
  */
 #define AssertReleaseMsg(expr, a)  \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
             RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
             RTAssertMsg2Weak a; \
@@ -1706,7 +1802,9 @@ RT_C_DECLS_END
  */
 #define AssertReleaseMsgReturn(expr, a, rc)  \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
             RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
             RTAssertMsg2Weak a; \
@@ -1723,7 +1821,9 @@ RT_C_DECLS_END
  */
 #define AssertReleaseMsgReturnVoid(expr, a)  \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
         { \
             RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
             RTAssertMsg2Weak a; \
@@ -1740,7 +1840,9 @@ RT_C_DECLS_END
  * @param   a       printf argument list (in parenthesis).
  */
 #define AssertReleaseMsgBreak(expr, a)  \
-    if (RT_UNLIKELY(!(expr))) \
+    if (RT_LIKELY(!!(expr))) \
+    { /* likely */ } \
+    else if (1) \
     { \
         RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
         RTAssertMsg2Weak a; \
@@ -1756,7 +1858,10 @@ RT_C_DECLS_END
  * @param   stmt    Statement to execute before break in case of a failed assertion.
  */
 #define AssertReleaseMsgBreakStmt(expr, a, stmt)  \
-    if (RT_UNLIKELY(!(expr))) { \
+    if (RT_LIKELY(!!(expr))) \
+    { /* likely */ } \
+    else if (1) \
+    { \
         RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
         RTAssertMsg2Weak a; \
         RTAssertReleasePanic(); \
@@ -1828,7 +1933,7 @@ RT_C_DECLS_END
  */
 #define AssertReleaseMsgFailed(a)  \
     do { \
-        RTAssertMsg1Weak((const char *)0, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
+        RTAssertMsg1Weak((const char *)0, __LINE__, __FILE__, RT_GCC_EXTENSION __PRETTY_FUNCTION__); \
         RTAssertMsg2Weak a; \
         RTAssertReleasePanic(); \
     } while (0)
@@ -1907,7 +2012,9 @@ RT_C_DECLS_END
  */
 #define AssertFatal(expr)  \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
             for (;;) \
             { \
                 RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
@@ -1923,7 +2030,9 @@ RT_C_DECLS_END
  */
 #define AssertFatalMsg(expr, a)  \
     do { \
-        if (RT_UNLIKELY(!(expr))) \
+        if (RT_LIKELY(!!(expr))) \
+        { /* likely */ } \
+        else \
             for (;;) \
             { \
                 RTAssertMsg1Weak(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
@@ -1939,7 +2048,7 @@ RT_C_DECLS_END
     do { \
         for (;;) \
         { \
-            RTAssertMsg1Weak((const char *)0, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
+            RTAssertMsg1Weak((const char *)0, __LINE__, __FILE__, RT_GCC_EXTENSION __PRETTY_FUNCTION__); \
             RTAssertReleasePanic(); \
         } \
     } while (0)
@@ -1953,7 +2062,7 @@ RT_C_DECLS_END
     do { \
         for (;;) \
         { \
-            RTAssertMsg1Weak((const char *)0, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
+            RTAssertMsg1Weak((const char *)0, __LINE__, __FILE__, RT_GCC_EXTENSION __PRETTY_FUNCTION__); \
             RTAssertMsg2Weak a; \
             RTAssertReleasePanic(); \
         } \
