@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2008-2013 Oracle Corporation
+ * Copyright (C) 2008-2015 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -21,6 +21,7 @@
 #include <iprt/alloca.h>
 #include <iprt/cpp/autores.h>
 #include <iprt/buildconfig.h>
+#include <iprt/getopt.h>
 #include <iprt/initterm.h>
 #include <iprt/mem.h>
 #include <iprt/message.h>
@@ -90,6 +91,7 @@ enum VBoxControlUsage
 #if !defined(VBOX_CONTROL_TEST)
     WRITE_CORE_DUMP,
 #endif
+    WRITE_LOG,
     TAKE_SNAPSHOT,
     SAVE_STATE,
     SUSPEND,
@@ -99,10 +101,10 @@ enum VBoxControlUsage
     USAGE_ALL = UINT32_MAX
 };
 
-static void usage(enum VBoxControlUsage eWhich = USAGE_ALL)
+static RTEXITCODE usage(enum VBoxControlUsage eWhich = USAGE_ALL)
 {
     RTPrintf("Usage:\n\n");
-    doUsage("print version number and exit", g_pszProgName, "[-v|--version]");
+    doUsage("print version number and exit", g_pszProgName, "[-V|--version]");
     doUsage("suppress the logo", g_pszProgName, "--nologo ...");
     RTPrintf("\n");
 
@@ -147,6 +149,8 @@ static void usage(enum VBoxControlUsage eWhich = USAGE_ALL)
     if (eWhich == WRITE_CORE_DUMP || eWhich == USAGE_ALL)
         doUsage("", g_pszProgName, "writecoredump");
 #endif
+    if (eWhich == WRITE_LOG || eWhich == USAGE_ALL)
+        doUsage("", g_pszProgName, "writelog [-n|--no-newline] [--] <msg>");
     if (eWhich == TAKE_SNAPSHOT || eWhich == USAGE_ALL)
         doUsage("", g_pszProgName, "takesnapshot");
     if (eWhich == SAVE_STATE || eWhich == USAGE_ALL)
@@ -159,10 +163,25 @@ static void usage(enum VBoxControlUsage eWhich = USAGE_ALL)
         doUsage("[command]", g_pszProgName, "help");
     if (eWhich == VERSION   || eWhich == USAGE_ALL)
         doUsage("", g_pszProgName, "version");
+
+    return RTEXITCODE_SUCCESS;
 }
 
 /** @} */
 
+
+/**
+ * Implementation of the '--version' option.
+ *
+ * @returns RTEXITCODE_SUCCESS
+ */
+static RTEXITCODE printVersion(void)
+{
+    RTPrintf("%sr%u\n", VBOX_VERSION_STRING, RTBldCfgRevision());
+    return RTEXITCODE_SUCCESS;
+}
+
+
 /**
  * Displays an error message.
  *
@@ -172,6 +191,7 @@ static void usage(enum VBoxControlUsage eWhich = USAGE_ALL)
  */
 static RTEXITCODE VBoxControlError(const char *pszFormat, ...)
 {
+    /** @todo prefix with current command. */
     va_list va;
     va_start(va, pszFormat);
     RTMsgErrorV(pszFormat, va);
@@ -180,6 +200,20 @@ static RTEXITCODE VBoxControlError(const char *pszFormat, ...)
 }
 
 
+/**
+ * Displays a getopt error.
+ *
+ * @returns RTEXITCODE_FAILURE.
+ * @param   ch          The RTGetOpt return value.
+ * @param   pValueUnion The RTGetOpt return data.
+ */
+static RTEXITCODE VBoxCtrlGetOptError(int ch, PCRTGETOPTUNION pValueUnion)
+{
+    /** @todo prefix with current command. */
+    return RTGetOptPrintError(ch, pValueUnion);
+}
+
+
 /**
  * Displays an syntax error message.
  *
@@ -189,6 +223,7 @@ static RTEXITCODE VBoxControlError(const char *pszFormat, ...)
  */
 static RTEXITCODE VBoxControlSyntaxError(const char *pszFormat, ...)
 {
+    /** @todo prefix with current command. */
     va_list va;
     va_start(va, pszFormat);
     RTMsgErrorV(pszFormat, va);
@@ -952,10 +987,10 @@ void getCustomModes(HKEY hkeyVideo)
 
         /* check if the mode is OK */
         if (   (xres > (1 << 16))
-            && (yres > (1 << 16))
-            && (   (bpp != 16)
-                || (bpp != 24)
-                || (bpp != 32)))
+            || (yres > (1 << 16))
+            || (   (bpp != 16)
+                && (bpp != 24)
+                && (bpp != 32)))
             break;
 
         /* add mode to table */
@@ -1063,10 +1098,10 @@ static RTEXITCODE handleAddCustomMode(int argc, char *argv[])
 
     /** @todo better check including xres mod 8 = 0! */
     if (   (xres > (1 << 16))
-        && (yres > (1 << 16))
-        && (   (bpp != 16)
-            || (bpp != 24)
-            || (bpp != 32)))
+        || (yres > (1 << 16))
+        || (   (bpp != 16)
+            && (bpp != 24)
+            && (bpp != 32)))
     {
         VBoxControlError("invalid mode specified!\n");
         return RTEXITCODE_FAILURE;
@@ -1176,7 +1211,7 @@ static RTEXITCODE getGuestProperty(int argc, char **argv)
     int rc = VINF_SUCCESS;
 
     rc = VbglR3GuestPropConnect(&u32ClientId);
-    if (!RT_SUCCESS(rc))
+    if (RT_FAILURE(rc))
         VBoxControlError("Failed to connect to the guest property service, error %Rrc\n", rc);
 
     /*
@@ -1220,7 +1255,7 @@ static RTEXITCODE getGuestProperty(int argc, char **argv)
         }
         if (VERR_TOO_MUCH_DATA == rc)
             VBoxControlError("Temporarily unable to retrieve the property\n");
-        else if (!RT_SUCCESS(rc) && (rc != VERR_NOT_FOUND))
+        else if (RT_FAILURE(rc) && rc != VERR_NOT_FOUND)
             VBoxControlError("Failed to retrieve the property value, error %Rrc\n", rc);
     }
 
@@ -1293,15 +1328,15 @@ static RTEXITCODE setGuestProperty(int argc, char *argv[])
     uint32_t u32ClientId = 0;
     int rc = VINF_SUCCESS;
     rc = VbglR3GuestPropConnect(&u32ClientId);
-    if (!RT_SUCCESS(rc))
+    if (RT_FAILURE(rc))
         VBoxControlError("Failed to connect to the guest property service, error %Rrc\n", rc);
-    if (RT_SUCCESS(rc))
+    else
     {
         if (pszFlags != NULL)
             rc = VbglR3GuestPropWrite(u32ClientId, pszName, pszValue, pszFlags);
         else
             rc = VbglR3GuestPropWriteValue(u32ClientId, pszName, pszValue);
-        if (!RT_SUCCESS(rc))
+        if (RT_FAILURE(rc))
             VBoxControlError("Failed to store the property value, error %Rrc\n", rc);
     }
 
@@ -1341,12 +1376,12 @@ static RTEXITCODE deleteGuestProperty(int argc, char *argv[])
      */
     uint32_t u32ClientId = 0;
     int rc = VbglR3GuestPropConnect(&u32ClientId);
-    if (!RT_SUCCESS(rc))
+    if (RT_FAILURE(rc))
         VBoxControlError("Failed to connect to the guest property service, error %Rrc\n", rc);
-    if (RT_SUCCESS(rc))
+    else
     {
         rc = VbglR3GuestPropDelete(u32ClientId, pszName);
-        if (!RT_SUCCESS(rc))
+        if (RT_FAILURE(rc))
             VBoxControlError("Failed to delete the property value, error %Rrc\n", rc);
     }
 
@@ -1484,7 +1519,7 @@ static RTEXITCODE waitGuestProperty(int argc, char **argv)
     int rc = VINF_SUCCESS;
 
     rc = VbglR3GuestPropConnect(&u32ClientId);
-    if (!RT_SUCCESS(rc))
+    if (RT_FAILURE(rc))
         VBoxControlError("Failed to connect to the guest property service, error %Rrc\n", rc);
 
     /*
@@ -1531,7 +1566,7 @@ static RTEXITCODE waitGuestProperty(int argc, char **argv)
         else if (rc == VERR_INTERRUPTED)
             VBoxControlError("The request timed out or was interrupted\n");
 #ifndef RT_OS_WINDOWS  /* Windows guests do not do this right */
-        else if (!RT_SUCCESS(rc) && (rc != VERR_NOT_FOUND))
+        else if (RT_FAILURE(rc) && rc != VERR_NOT_FOUND)
             VBoxControlError("Failed to get a notification, error %Rrc\n", rc);
 #endif
     }
@@ -1615,7 +1650,7 @@ static RTEXITCODE listSharedFolders(int argc, char **argv)
 
     uint32_t u32ClientId;
     int rc = VbglR3SharedFolderConnect(&u32ClientId);
-    if (!RT_SUCCESS(rc))
+    if (RT_FAILURE(rc))
         VBoxControlError("Failed to connect to the shared folder service, error %Rrc\n", rc);
     else
     {
@@ -1722,6 +1757,69 @@ static RTEXITCODE handleDpc(int argc, char *argv[])
 #endif /* VBOX_WITH_DPC_LATENCY_CHECKER */
 
 
+/**
+ * @callback_method_impl{FNVBOXCTRLCMDHANDLER, Command: writelog}
+ */
+static RTEXITCODE handleWriteLog(int argc, char *argv[])
+{
+    static const RTGETOPTDEF s_aOptions[] =
+    {
+        { "--no-newline", 'n', RTGETOPT_REQ_NOTHING },
+    };
+    bool fNoNewline = false;
+
+    RTGETOPTSTATE GetOptState;
+    int rc = RTGetOptInit(&GetOptState, argc, argv, s_aOptions, RT_ELEMENTS(s_aOptions),
+                          0 /*iFirst*/, RTGETOPTINIT_FLAGS_OPTS_FIRST);
+    if (RT_SUCCESS(rc))
+    {
+        RTGETOPTUNION   ValueUnion;
+        int             ch;
+        while ((ch = RTGetOpt(&GetOptState, &ValueUnion)) != 0)
+        {
+            switch (ch)
+            {
+                case VINF_GETOPT_NOT_OPTION:
+                {
+                    size_t cch = strlen(ValueUnion.psz);
+                    if (   fNoNewline
+                        || (cch > 0 && ValueUnion.psz[cch - 1] == '\n') )
+                        rc = VbglR3WriteLog(ValueUnion.psz, cch);
+                    else
+                    {
+                        char *pszDup = (char *)RTMemDupEx(ValueUnion.psz, cch, 2);
+                        if (RT_SUCCESS(rc))
+                        {
+                            pszDup[cch++] = '\n';
+                            pszDup[cch]   = '\0';
+                            rc = VbglR3WriteLog(pszDup, cch);
+                            RTMemFree(pszDup);
+                        }
+                        else
+                            rc = VERR_NO_MEMORY;
+                    }
+                    if (RT_FAILURE(rc))
+                        return VBoxControlError("VbglR3WriteLog: %Rrc", rc);
+                    break;
+                }
+
+                case 'n':
+                    fNoNewline = true;
+                    break;
+
+                case 'h': return usage(WRITE_LOG);
+                case 'V': return printVersion();
+                default:
+                    return VBoxCtrlGetOptError(ch, &ValueUnion);
+            }
+        }
+    }
+    else
+        return VBoxControlError("RTGetOptInit: %Rrc", rc);
+    return RTEXITCODE_SUCCESS;
+}
+
+
 /**
  * @callback_method_impl{FNVBOXCTRLCMDHANDLER, Command: takesnapshot}
  */
@@ -1765,8 +1863,7 @@ static RTEXITCODE handleVersion(int argc, char *argv[])
 {
     if (argc)
         return VBoxControlSyntaxError("getversion does not take any arguments");
-    RTPrintf("%sr%u\n", VBOX_VERSION_STRING, RTBldCfgRevision());
-    return RTEXITCODE_SUCCESS;
+    return printVersion();
 }
 
 /**
@@ -1812,6 +1909,7 @@ struct COMMANDHANDLER
 #ifdef VBOX_WITH_DPC_LATENCY_CHECKER
     { "dpc",                    handleDpc },
 #endif
+    { "writelog",               handleWriteLog },
     { "takesnapshot",           handleTakeSnapshot },
     { "savestate",              handleSaveState },
     { "suspend",                handleSuspend },
@@ -1857,7 +1955,7 @@ int main(int argc, char **argv)
            )
         {
             /* Print version number, and do nothing else. */
-            RTPrintf("%sr%u\n", VBOX_VERSION_STRING, RTBldCfgRevision());
+            printVersion();
             fOnlyInfo = true;
             fShowLogo = false;
             done = true;
