@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2007-2010 Oracle Corporation
+ * Copyright (C) 2007-2015 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -97,6 +97,14 @@ VBGLR3DECL(int)     VbglR3GuestPropRead(uint32_t u32ClientId,
     return VINF_SUCCESS;
 }
 
+VBGLR3DECL(int)     VbglR3GuestPropDelete(uint32_t u32ClientId,
+                                          const char *pszName)
+{
+    RTPrintf("Called DEL_PROP, client %d, name %s...\n",
+             u32ClientId, pszName);
+    return VINF_SUCCESS;
+}
+
 struct VBGLR3GUESTPROPENUM
 {
     uint32_t u32;
@@ -191,3 +199,10 @@ VBGLR3DECL(int)     VbglR3GuestPropWait(uint32_t u32ClientId,
 }
 
 #endif
+
+VBGLR3DECL(int) VbglR3WriteLog(const char *pch, size_t cch)
+{
+    NOREF(pch); NOREF(cch);
+    return VINF_SUCCESS;
+}
+
