@@ -1,10 +1,10 @@
-/* $Revision: 88366 $ */
+/* $Id: HGCMInternal.cpp $ */
 /** @file
  * VBoxGuestLib - Host-Guest Communication Manager internal functions, implemented by VBoxGuest
  */
 
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2015 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -207,7 +207,7 @@ static int vbglR0HGCMInternalPreprocessCall(VBoxGuestHGCMCallInfo const *pCallIn
                 break;
 
             case VMMDevHGCMParmType_64bit:
-                Log4(("GstHGCMCall: parm=%u type=64bit: %#018x\n", iParm, pSrcParm->u.value64));
+                Log4(("GstHGCMCall: parm=%u type=64bit: %#018RX64\n", iParm, pSrcParm->u.value64));
                 break;
 
             case VMMDevHGCMParmType_PageList:
@@ -336,7 +336,7 @@ static int vbglR0HGCMInternalPreprocessCall(VBoxGuestHGCMCallInfo const *pCallIn
                          */
                         /** @todo A more efficient strategy would be to combine buffers. However it
                          *        is probably going to be more massive than the current code, so
-                         *        it can wait till later.   */
+                         *        it can wait till later. */
                         bool fCopyIn = pSrcParm->type != VMMDevHGCMParmType_LinAddr_Out
                                     && pSrcParm->type != VMMDevHGCMParmType_LinAddr_Locked_Out;
                         if (cb <= PAGE_SIZE / 2 - 16)
